#!/usr/bin/env python3

import scipy.io as sio

Assoc_eddies_Anti_max = sio.loadmat("Association_eddies_Anti_max_v6.mat",
                                          squeeze_me=True)
Assoc_eddies_Cyclo_max = sio.loadmat("Association_eddies_Cyclo_max_v6.mat",
                                          squeeze_me=True)
Nanti = sio.loadmat("Parameters_Anticyclonic_Eddies_v6.mat", 
                    squeeze_me = True, variable_names = ["Nanti"])["Nanti"]
Ncyclo = sio.loadmat("Parameters_Cyclonic_Eddies_v6.mat", squeeze_me = True,
                     variable_names = ["Ncyclo"])["Ncyclo"]

def convert_FEI(orientation, eddy_index_Matlab):
    """Convert from Matlab eddy identification to eddy
    identification in shapefiles.

    We assume that the shapefiles are created by the script
    "mat_to_shapefiles.py". This script processes a single snapshot
    and all the anticyclones are processed before all the cyclones."""

    if orientation == "anticyclone":
        if eddy_index_Matlab <= Nanti[0]:
            date = Assoc_eddies_Anti_max["date_num"][0]
            eddy_index_F = eddy_index_Matlab
        else:
            date = Assoc_eddies_Anti_max["date_num"][1]
            eddy_index_F = eddy_index_Matlab - 100
    else:
        if eddy_index_Matlab <= Ncyclo[0]:
            date = Assoc_eddies_Anti_max["date_num"][0]
            eddy_index_F = Nanti[0] + eddy_index_Matlab
        else:
            date = Assoc_eddies_Anti_max["date_num"][1]
            eddy_index_F = Nanti[1] + eddy_index_Matlab - 100

    return date, eddy_index_F

def print_edgelist(orientation, total_number, id_child):
    for i1 in range(total_number):
        predecessor = (orientation, i1 + 1)
        try:
            for eddy_index_Matlab in id_child[i1]:
               print(*convert_FEI(*predecessor), 
                     *convert_FEI(orientation, eddy_index_Matlab))
        except TypeError:
            print(*convert_FEI(*predecessor), 
                   *convert_FEI(orientation, id_child[i1]))

print_edgelist("anticyclone", Nanti[0], Assoc_eddies_Anti_max["id_child"][:, 0])
print_edgelist("cyclone", Ncyclo[0], Assoc_eddies_Cyclo_max["id_child"][:, 0])
