#!/usr/bin/env python3

import shapefile
import numpy as np

def write(matlab_data):
    """matlab_data is a dict."""

    with shapefile.Writer("Snapshot/extremum", 
                          shapeType = shapefile.POINT) as w_extr, \
      shapefile.Writer("Snapshot/outermost_contour", 
                       shapeType = shapefile.POLYGON) as w_outer, \
      shapefile.Writer("Snapshot/max_speed_contour", 
                       shapeType = shapefile.POLYGON) as w_max_speed:
        w_extr.field("ssh", "N", 13, 6)
        w_extr.field("date_index", "N", 6)
        w_extr.field("eddy_index", "N", 5)
        w_extr.field("interpolat", "N", 1)
        w_extr.field("cyclone", "N", 1)
        w_extr.field("valid", "N", 1)
        w_extr.field("speed", "N", 13, 6)

        w_outer.field("r_eq_area", "N", 10, 4)
        w_outer.field("ssh", "N", 13, 6)
        w_outer.field("date_index", "N", 6)
        w_outer.field("eddy_index", "N", 5)
        w_outer.field("radius4", "N", 2)

        w_max_speed.field("r_eq_area", "N", 10, 4)
        w_max_speed.field("ssh", "N", 13, 6)
        w_max_speed.field("date_index", "N", 6)
        w_max_speed.field("eddy_index", "N", 5)

        eddy_index = 0

        for eddy_orientation in ["Anticyclonic_Cell", "Cyclonic_Cell"]:
            # matlab_data[eddy_orientation] is a numpy.ndarray with
            # shape (number of eddies, 21).

            for eddy in matlab_data[eddy_orientation]:
                # eddy is a numpy.ndarray with shape (21,). The
                # description of each element of eddy is given by
                # matlab_data["Fields"]. Note that amplitudes in
                # eddy[7] and eddy[11] are positive regardless of eddy
                # orientation.

                eddy_index += 1
                cyclone = eddy_orientation == "Cyclonic_Cell"

                w_extr.point(eddy[0], eddy[1])
                i = np.argwhere(matlab_data["X"] == eddy[0]).item()
                j = np.argwhere(matlab_data["Y"] == eddy[1]).item()
                speed = 1e4 if np.isnan(eddy[12]) else eddy[12]
                w_extr.record(ssh = matlab_data["ADT"][i, j], 
                              date_index = matlab_data["date_num"],
                              eddy_index = eddy_index, interpolat = 0, 
                              cyclone = cyclone, valid = 1, speed = speed)

                if cyclone:
                    ssh = matlab_data["ADT"][i, j] + eddy[7]
                else:
                    ssh =  matlab_data["ADT"][i, j] - eddy[7]

                w_outer.record(r_eq_area = eddy[6], ssh = ssh, 
                               date_index = matlab_data["date_num"], 
                               eddy_index = eddy_index, radius4 = - 1)
                polyline = np.stack((eddy[4], eddy[5]), axis = 1)
                w_outer.poly([polyline])

                if cyclone:
                    ssh = matlab_data["ADT"][i, j] + eddy[11]
                else:
                    ssh =  matlab_data["ADT"][i, j] - eddy[11]

                w_max_speed.record(r_eq_area = eddy[10], ssh = ssh, 
                                   date_index = matlab_data["date_num"], 
                                   eddy_index = eddy_index)
                polyline = np.stack((eddy[8], eddy[9]), axis = 1)
                w_max_speed.poly([polyline])

if __name__ == "__main__":
    import sys
    import scipy.io as sio
    
    if len(sys.argv) != 2: sys.exit("Required argument: .mat file")
    matlab_data = sio.loadmat(sys.argv[1], squeeze_me = True)
    write(matlab_data)
